/* This is the console executable, that makes use of the BullCow class.
This acts as the view in a MVC pattern, and is responsible for all
user interaction. For game logic see the FBullCowGameClass
*/
#pragma once

#include <iostream>
#include <string>
#include "FBullCowGame.h"

//To make syntax friendly to the Unreal engine
using FText = std::string;
using int32 = int;

// Function prototypes as outside a class
void PrintIntro();
void PlayGame();
FText GetValidGuess();
bool AskToPlayAgain();
void PrintGameSummary();

FBullCowGame BCGame;  //Instantiate a new game, which are re-used across plays.

//The entry point for our application
int main()
{
	bool bPlayAgain = false;
	do {
		PrintIntro();
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	} 
	while (bPlayAgain);
	return 0; //Exit the game
}

void PrintIntro() 
{
	std::cout << " \\|/          (__)\n";
	std::cout << "      `\\------(oo)\n";
	std::cout << "       ||     (__)\n";
	std::cout << "       ||w--||      \\|/ \n";
	std::cout << "   \\|/";
	std::cout << "\n\n Welcome to Bulls and Cows, a fun word game.\n";
	std::cout << "Could you guess my " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram I'm thinking of?\n";
	std::cout << std::endl;
	return;
}

//Plays one game until completion
void PlayGame()
{
	BCGame.Reset();

	int32 MaxTries = BCGame.GetMaxTries();
	
	//Loop asking for guess while game is 
	//NOT won and there are still tries.
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries) {
		FText Guess = GetValidGuess();
		
 		//Summit valid guess in the game, and receive counts
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);

		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << ". Cows = " << BullCowCount.Cows << "." << "\n\n";
	}
	
	PrintGameSummary();
	return;
}

// Loop continually until the user gives a valid guess
FText GetValidGuess()
{
	FText Guess = "";
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	do {
		//Get a guess from the player
		int32 CurrentTry = BCGame.GetCurrentTry();
		int32 MaxTries = BCGame.GetMaxTries();
		std::cout << "Try " << CurrentTry << " of " << MaxTries;
		std::cout << ". Enter your guess: ";
		std::getline(std::cin, Guess);

		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n\n";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Please enter using lowercase letters.\n\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter a word without repeating letters.\n\n";
			break;
		default:
			//Assume that the guess is valid
			break;
		}
	} while (Status != EGuessStatus::OK); // Keep looping until we got no errors.
	return Guess;
}

bool AskToPlayAgain()
{
	std::cout << "Do you want to play again with the same hidden word (y/n)? ";
	FText Response = "";
	std::getline(std::cin, Response);
	return (Response[0] == 'y') || (Response[0] == 'Y');
}

//Conclude the game
void PrintGameSummary()
{
	if (BCGame.IsGameWon())
	{
		std::cout << "Congratulations! You guessed the secret word.\n";
	}
	else
	{
		std::cout << "Oooh! Too bad! Better luck next time.\n";
	}
	return;
}